// startup.cpp - passing binary data to a child process

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

// manifest constants from MSDOS.H:
#define FOPEN       0x01    /* file handle open */
#define FEOFLAG     0x02    /* end of file has been encountered */
#define FCRLF       0x04    /* CR-LF across read buffer */
#define FPIPE       0x08    /* file handle refers to a pipe */
#define FNOINHERIT  0x10    /* file handle opened _O_NOINHERIT */
#define FAPPEND     0x20    /* file handle opened O_APPEND */
#define FDEV        0x40    /* file handle refers to device */
#define FTEXT       0x80    /* file handle is in text mode */

#ifdef __BORLANDC__
  #define NUM_HANDLES   0
#else
  #define NUM_HANDLES   3
#endif

HANDLE SpawnWithInfo(LPTSTR szCommand,
            HANDLE hStdin, HANDLE hStdout, HANDLE hStderr,
            void *pAuxInfo, WORD cbAuxInfo)
{
    size_t buf_len = sizeof (DWORD) + NUM_HANDLES + \
            NUM_HANDLES * sizeof(HANDLE) + cbAuxInfo;
    unsigned char *buf = new unsigned char[buf_len];
    STARTUPINFO si;

    ZeroMemory(&si, sizeof(si));
    si.cb = sizeof(si);
    si.cbReserved2 = (WORD) buf_len;
    si.lpReserved2 = buf;

    *(DWORD *)buf = NUM_HANDLES;
    unsigned char * pFlags = buf + sizeof (DWORD);
    HANDLE * pHandles = (HANDLE*)(pFlags + NUM_HANDLES);

    if (pAuxInfo != NULL)
        CopyMemory(pHandles + NUM_HANDLES, pAuxInfo, cbAuxInfo);

#if NUM_HANDLES > 0
    pFlags[0] = FOPEN | FTEXT;
    pHandles[0] = hStdin;
    pFlags[1] = FOPEN | FTEXT;
    pHandles[1] = hStdout;
    pFlags[2] = FOPEN | FTEXT;
    pHandles[2] = hStderr;
#else
    (void)UNREFERENCED_PARAMETER(hStdin);
    (void)UNREFERENCED_PARAMETER(hStdout);
    (void)UNREFERENCED_PARAMETER(hStderr);
#endif

    PROCESS_INFORMATION pi;
    if ( ! CreateProcess(NULL, szCommand, NULL, NULL,
        TRUE, 0, NULL, NULL, &si, &pi)) {
        DWORD err = GetLastError();
        delete buf;
        SetLastError(err);
        return INVALID_HANDLE_VALUE;
    }

    delete buf;
    CloseHandle(pi.hThread);
    return pi.hProcess;
}
// end of file