/****************************************************
** This app demonstrates how to use PlayMidiSound
** to play back resources and files under Win32
**
** Written by Davide Ficano for WDJ
*****************************************************/
#define STRICT

#include <windows.h>
#include <windowsx.h>
#include "resource.h"
#include "midires.h"

// hInstance of application
HINSTANCE g_hInst;
// Edit window
HWND g_hwndEdit;

void PlayResource(HWND hwnd, LPCTSTR lpMidiRes, HWND hwndEdit);
void PlayFile(HWND hwnd, HWND hwndEdit);
BOOL GetMidiFilename(HWND hwnd, LPTSTR szFilename);

BOOL CALLBACK DlgMidiProc(HWND hwndDlg, UINT uMsg,
                          WPARAM wParam, LPARAM lParam);



int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpszCmdLine, int cmdShow)
{
    g_hInst = hInstance;
    
    return DialogBox(hInstance, _T("IDD_MAIN"),
                     NULL, (DLGPROC)DlgMidiProc);
}

BOOL CALLBACK DlgMidiProc(HWND hwndDlg, UINT uMsg,
                          WPARAM wParam, LPARAM lParam)
{
    switch (uMsg)
    {
        case WM_INITDIALOG:
            CheckDlgButton(hwndDlg, IDC_ASYNC, 1);
            g_hwndEdit = GetDlgItem(hwndDlg, IDC_MESSAGES);
            break;
        case WM_DESTROY:
            // Always stop playing when destroy window
            PlayMidiSound(NULL, NULL, 0);
            break;
        case WM_COMMAND:
            switch (GET_WM_COMMAND_ID(wParam, lParam))
            {
                case IDC_PLAY:
                    PlayResource(hwndDlg,
                                 MAKEINTRESOURCE(IDR_MIDI1),
                                 g_hwndEdit);
                    break;
                case IDC_PLAYFILE:
                    PlayFile(hwndDlg, g_hwndEdit);
                    break;
                case IDC_STOP:
                    PlayMidiSound(NULL, NULL, 0);
                    Edit_ReplaceSel(g_hwndEdit,
                                   _T("Stop playing\r\n"));
                    break;
                case IDCANCEL:
                    EndDialog(hwndDlg, FALSE);
                    return FALSE;
                default:
                    break;
            }
            break;
    }
    return FALSE;
}

void PlayResource(HWND hwnd, LPCTSTR lpMidiRes, HWND hwndEdit)
{
    BOOL bAsync;
    LPCTSTR pszMsg;
    
    bAsync = IsDlgButtonChecked(hwnd, IDC_ASYNC) ? SND_ASYNC : 0;
    
    if (PlayMidiSound(lpMidiRes, g_hInst, SND_RESOURCE | bAsync))
        pszMsg = _T("Start playing resource\r\n");
    else
        pszMsg = _T("Unable to play resource\r\n");
    Edit_ReplaceSel(hwndEdit, pszMsg);
}

void PlayFile(HWND hwnd, HWND hwndEdit)
{
    BOOL bAsync;
    TCHAR szMidiName[MAX_PATH];
    LPCTSTR pszMsg;
    
    if (!GetMidiFilename(hwnd, szMidiName))
        return;
    
    bAsync = IsDlgButtonChecked(hwnd, IDC_ASYNC) ? SND_ASYNC : 0;
    
    if (PlayMidiSound(szMidiName, g_hInst, SND_FILENAME | bAsync))
        pszMsg = _T("Start playing file\r\n");
    else
        pszMsg = _T("Unable to play file\r\n");
    Edit_ReplaceSel(hwndEdit, pszMsg);
}

BOOL GetMidiFilename(HWND hwnd, LPTSTR szFilename)
{
    OPENFILENAME ofn;
    static TCHAR szFilters[] = _T("Midi files (*.mid;*.rmi)\0"
                                  "*.mid;*.rmi\0"
                                  "All files (*.*)\0*.*\0"
                                  "\0");
    
    szFilename[0] = _T('\0');
    
    memset(&ofn, 0, sizeof (OPENFILENAME));
    
    ofn.lStructSize = sizeof(OPENFILENAME);
    ofn.hwndOwner   = hwnd;
    ofn.lpstrFilter = szFilters;
    ofn.lpstrFile   = szFilename;
    ofn.nMaxFile    = MAX_PATH;
    ofn.Flags       = OFN_PATHMUSTEXIST;
    
    return GetOpenFileName(&ofn);
}
