// wdjsrvc.c

#include <windows.h>
#include <stdio.h>

// prototypes

BOOL NotifySCM(DWORD, DWORD, DWORD);
VOID WINAPI ServiceMain(DWORD, LPTSTR *);
VOID WINAPI ServiceHandler(DWORD);
DWORD WINAPI MainServiceThread(LPDWORD);
LRESULT CALLBACK ServiceDlgProc(HWND, UINT, WPARAM, LPARAM);

// globals

HANDLE  hDoneEvent = NULL, hThread = NULL;
DWORD   dwCurrentState;
SERVICE_STATUS_HANDLE  hService;
TCHAR gszInputDesktop[MAX_PATH];
HDESK ghDesktop;

//--------------------------------------------------------------
void main(void)
{
   SERVICE_TABLE_ENTRY ServiceTable[] = {
      {TEXT("WDJSrvc"), (LPSERVICE_MAIN_FUNCTION)ServiceMain},
      {NULL, NULL}
   };

   // connect to  the service control manager
   StartServiceCtrlDispatcher(ServiceTable);
}

//--------------------------------------------------------------
VOID WINAPI ServiceMain(DWORD dwArgc, LPTSTR *lpszArgv)
{
   DWORD   ThreadId;

   if (!(hService = RegisterServiceCtrlHandler(TEXT("WDJSrvc"),
         (LPHANDLER_FUNCTION)ServiceHandler))) {
       return;
   }

   NotifySCM(SERVICE_START_PENDING, 0, 1);

   if (!(hDoneEvent = CreateEvent(NULL, FALSE, FALSE, NULL))) {
       return;
   }

   NotifySCM(SERVICE_START_PENDING, 0, 2);

   if (!(hThread = CreateThread(0, 0,
         (LPTHREAD_START_ROUTINE)MainServiceThread, 0, 0,
         &ThreadId))) {
      CloseHandle(hDoneEvent);
      return;
   }

   NotifySCM(SERVICE_RUNNING, 0, 0);

   WaitForSingleObject(hDoneEvent, INFINITE);

   CloseHandle(hThread);
   CloseHandle(hDoneEvent);
   return;
}

//--------------------------------------------------------------
VOID WINAPI ServiceHandler(DWORD fdwControl)
{
   switch(fdwControl) {

      case SERVICE_CONTROL_STOP:
         NotifySCM(SERVICE_STOP_PENDING, 0, 1);
         SetEvent(hDoneEvent);
         NotifySCM(SERVICE_STOPPED, 0, 0);
         break;

      case SERVICE_CONTROL_INTERROGATE:
         NotifySCM(dwCurrentState, 0, 0);
         break;

      case SERVICE_CONTROL_SHUTDOWN:
         break;
   }
}

//-------------------------------------------------------------------
BOOL NotifySCM(DWORD dwState, DWORD dwWin32ExitCode, DWORD dwProgress)
{
   SERVICE_STATUS ServiceStatus;

   ServiceStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
   ServiceStatus.dwCurrentState = dwCurrentState = dwState;
   ServiceStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP |
      SERVICE_ACCEPT_SHUTDOWN;
   ServiceStatus.dwWin32ExitCode = dwWin32ExitCode;
   ServiceStatus.dwServiceSpecificExitCode = 0;
   ServiceStatus.dwCheckPoint = dwProgress;
   ServiceStatus.dwWaitHint = 5000;

   return SetServiceStatus(hService, &ServiceStatus);
}

//-------------------------------------------------------------------
DWORD WINAPI MainServiceThread(LPDWORD ThreadParam)
{
   HDESK hOldDesktop;
   char szName[MAX_PATH], szDbg[MAX_PATH];
   int i;
   BOOL Done = FALSE;

   OutputDebugString("WDJSRVC: In MainServiceThread\n");

   // Save the service process's current desktop.
      
   hOldDesktop = GetThreadDesktop(GetCurrentThreadId());
   GetUserObjectInformation(hOldDesktop, UOI_NAME, szName, 
                            MAX_PATH * sizeof(TCHAR), &i);

   // Retrieve a handle (and name) of the current input desktop.
 
   ghDesktop = OpenInputDesktop(0, FALSE, DESKTOP_CREATEWINDOW);
   GetUserObjectInformation(ghDesktop, UOI_NAME, gszInputDesktop, 
                            MAX_PATH * sizeof(TCHAR), &i);

   wsprintf(szDbg, TEXT("WDJSRVC: desktop=%s, input desktop=%s\n"), 
            gszInputDesktop, szName);
   OutputDebugString(szDbg);

   // Redisplay the dialog box as long as it returns FALSE.

   while (!Done) {
      SetThreadDesktop(ghDesktop);
      Done = DialogBox(GetModuleHandle(NULL),
                   TEXT("SERVICE_DIALOG"), NULL, ServiceDlgProc);
   }

   // Restore the service process's original desktop.

   SetThreadDesktop(hOldDesktop);

   OutputDebugString("WDJSRVC: DialogBox closed\n");

   while (TRUE) {
      // Do more service processing...
      Sleep(10000);
   }
   return TRUE;
}

//-------------------------------------------------------------------
LRESULT CALLBACK ServiceDlgProc(HWND hDlg, UINT msg,
   WPARAM wParam, LPARAM lParam)
{
   TCHAR szDbg[MAX_PATH], szName[MAX_PATH];
   HDESK hDesktop;
   int i;

   switch (msg) {
      case WM_INITDIALOG:
         SetTimer(hDlg, 1, 5000, NULL);
         break;

      case WM_TIMER:

         // Get the current input desktop.
		 
         hDesktop = OpenInputDesktop(0,FALSE,DESKTOP_CREATEWINDOW);
         GetUserObjectInformation(hDesktop, UOI_NAME, szName, 
                                  MAX_PATH * sizeof(TCHAR), &i);

         // Compare the current input desktop with the previous
         // input desktop. If different, dismiss this dialog. The 
         // main thread will set the service process's desktop to 
         // the new input desktop and recreate the dialog box.

         if (lstrcmp(szName, gszInputDesktop) != 0) {
            wsprintf(szDbg,TEXT("WDJSRVC: Input desktop = %s\n"),
                     szName);
            OutputDebugString(szDbg);
            lstrcpy(gszInputDesktop, szName);
            ghDesktop = hDesktop;
            EndDialog(hDlg, FALSE);
         }
         break;

      case WM_COMMAND:
         switch(LOWORD(wParam)) {
            case IDOK:
               KillTimer(hDlg, 1);
               EndDialog(hDlg, TRUE);
               return TRUE;
      }
      break;
   }
   return FALSE;
}


